/*
 * Ben NanoNote board late initialisation, based on uboot-xburst and xburst-tools.
 *
 * Copyright (C) 2000-2009 Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
 * Copyright (C) 2006 Ingenic Semiconductor, <jlwei@ingenic.cn>
 * Copyright (C) Xiangfu Liu <xiangfu.z@gmail.com>
 * Copyright (C) 2015, 2016, 2017 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "board.h"
#include "nanonote.h"

/* Later initialisation functions. */

void gpio_init2()
{
	/* Initialise LCD pins. */

	__gpio_as_slcd_8bit();

	/* Initialise MSC pins. */

	__gpio_as_msc();

	/* Initialise other pins. */

	/* Enable TP4, TP5 as UART0. */

	__gpio_jtag_to_uart0();

	__gpio_as_input(GPIO_POWER);
	__gpio_enable_pull(GPIO_POWER);

	__gpio_as_output(GPIO_AUDIO_POP);
	__gpio_set_pin(GPIO_AUDIO_POP);

	__gpio_as_output(GPIO_LCD_CS);
	__gpio_clear_pin(GPIO_LCD_CS);

	__gpio_as_output(GPIO_AMP_EN);
	__gpio_clear_pin(GPIO_AMP_EN);

	__gpio_as_output(GPIO_SDPW_EN);
	__gpio_disable_pull(GPIO_SDPW_EN);
	__gpio_clear_pin(GPIO_SDPW_EN);

	__gpio_as_input(GPIO_SD_DETECT);
	__gpio_disable_pull(GPIO_SD_DETECT);

	__gpio_as_input(GPIO_USB_DETECT);
	__gpio_enable_pull(GPIO_USB_DETECT);

	/* Employ a PWM channel for the buzzer. */

	__gpio_as_pwm4();
}

void cpm_init()
{
	__cpm_stop_ipu();
	__cpm_stop_cim();
	__cpm_stop_i2c();
	__cpm_stop_ssi();
	__cpm_stop_uart1();
	__cpm_stop_sadc();
	__cpm_stop_uhc();
	__cpm_stop_udc();
	__cpm_stop_aic1();
/*	__cpm_stop_aic2();*/
}

void rtc_init()
{
	while ( !__rtc_write_ready());
	__rtc_enable_alarm();	/* enable alarm */

	while ( !__rtc_write_ready());
	REG_RTC_RGR   = 0x00007fff; /* type value */

	while ( !__rtc_write_ready());
	REG_RTC_HWFCR = 0x0000ffe0; /* Power on delay 2s */

	while ( !__rtc_write_ready());
	REG_RTC_HRCR  = 0x00000fe0; /* reset delay 125ms */
}

/* Timer routines. */

unsigned long timestamp;
unsigned long lastdec;

/*
 * Timer without interrupts.
 */

void timer_init()
{
	__tcu_disable_pwm_output(TIMER_CHAN);
	__tcu_select_extalclk(TIMER_CHAN);
	__tcu_select_clk_div256(TIMER_CHAN);
	__tcu_set_count(TIMER_CHAN, 0);
	__tcu_set_half_data(TIMER_CHAN, 0);
	__tcu_set_full_data(TIMER_CHAN, TIMER_FDATA);

	__tcu_mask_half_match_irq(TIMER_CHAN);
	__tcu_mask_full_match_irq(TIMER_CHAN);
	__tcu_start_timer_clock(TIMER_CHAN);
	__tcu_start_counter(TIMER_CHAN);

	__cpm_start_tcu();

	lastdec = 0;
	timestamp = 0;
}

/* Timer interrupt activation. */

void timer_init_irq()
{
	__tcu_unmask_full_match_irq(TIMER_CHAN);
	__tcu_clear_full_match_flag(TIMER_CHAN);
	__intc_unmask_irq(TIMER_CHAN_IRQ);
}

void timer_clear()
{
	__intc_ack_irq(TIMER_CHAN_IRQ);
	__tcu_clear_full_match_flag(TIMER_CHAN);
}

/* GPIO interrupt activation. */

void gpio_init_irq()
{
	__gpio_as_irq_low_level(GPIO_POWER);
	__intc_unmask_irq(GPIO_IRQ);
}

int gpio_have_irq(uint8_t gpio)
{
	return (REG_GPIO_PXFLG(gpio / 32) & (1 << (gpio % 32)));
}

/* Board startup detection. */

int is_started()
{
	return REG_CPM_CLKGR != 0;
}
